
//////////////////////////////////////////////////////////////////////////////////////////////////
//	Heap
//	Copyleft by Stefanie Wiegand 2007
//	function "void print(Heap h)" by Hong Phuc Bui
//	while-condition and most of the comments by D. Weber 
//	Version 2.0
//	heap (in array) visualized
//	g++ -Wall -pedantic printedheap.cpp -o heap
//	Rueckgabewert bei erfolgreicher Ausfuehrung: 0
//////////////////////////////////////////////////////////////////////////////////////////////////

#include <cmath>
#include <iostream>
using namespace std;

const int MAX_ELEMENTS=50;

struct Heap
{
	int size;
	int element[MAX_ELEMENTS];
};

bool isempty(Heap h);
bool isfull(Heap h);
void insert(Heap& rh, int new_element);
int delete_max(Heap& rh);
void print_array(Heap h);
void print(Heap &heap);

void cls();

int main()
{
Heap h;
Heap& rh = h;
h.size = 0;
char choice = 'a';
int new_element = 0;
int deleted = 0;

do  // loop until user terminates by pressing 'e'
{
	cls();
	print(h); // print state of heap
	cout << "Press..." << endl << "i to insert" << endl;
	cout << "d to delete" << endl << "e to exit" << endl;
	cin >> choice; // user choice
	switch (choice)
	{
		case 'i': // insert
		{
			if (isfull(h) == false)
			{
				cout << "Please insert element: " << endl;
				cin >> new_element;
				insert (rh, new_element);
			} else
			{
				cout << "Heap is full, can't insert element!" << endl;
			}
		} break;
		case 'd': // delete max
		{
			if (isempty(h) == false)
			{
				deleted = delete_max (rh);
				cout << "Delete Maximum (" << deleted << ")" << endl;
			} else
			{
				cout << "Can't delete from empty heap!" << endl;
			}
		} break;
		case 'e': // exit
		{
			cout << "Exiting now!" << endl;
		} break;
		default: // input error
		{
			cout << "Wrong input, please try again!" << endl;
		} break;
	}
}while (choice != 'e');


return 0;
}


// heap functions

bool isempty(Heap h)
{
	if (h.size == 0)
	{
		return true;
	} else
	{
		return false;
	}
}

bool isfull(Heap h)
{
	if (h.size == MAX_ELEMENTS)
	{
		return true;
	} else
	{
		return false;
	}
}

void insert(Heap& rh, int new_element)
{
	int position;
	int tmp = 0;

	rh.size++; 				// one more element 
	rh.element[rh.size] = new_element;	// element now in heap
	position=rh.size;

	// loop until root reached or heap ok
	while ((int(position/2) > 0) && (rh.element[position] > rh.element[int(position/2)]))
	{
#ifdef DEBUG
		cout << "Heapcondition violated!" << endl << "swapping " << endl;
		cout << rh.element[position] << " with " << rh.element[int(position/2)] << endl;
#endif
		tmp = rh.element[position]; 	// swap with parent
		rh.element[position] = rh.element[int(position/2)];
		rh.element[int(position/2)] = tmp;
		position = int(position/2);	// up one level 	
	}
}

int delete_max(Heap& rh)
{
	int deleted = rh.element[1];
	rh.element[1] = rh.element[rh.size];
	rh.size--;
	int tmp = 0;
	int position = 1;
	int l = 0;	// left child
	int r = 0;	// right child
	int max = 0;	// pos of max child
	while (2*position <= rh.size)	// a child exists
	{
	
		l = position*2;
		r = (position*2)+1;
		if (r <= rh.size)	// right child exists
		{
			if ( rh.element[l] < rh.element[r])
			{
				max = r;		
			} else
			{
				max = l;
			}
		} else
		{
			max = l;
		}
		if ( rh.element[position] < rh.element[max])	//heap condition violated?
		{ // yes
			tmp = rh.element[position]; 
			rh.element[position] = rh.element[max];
			rh.element[max] = tmp;
			position = max;
		} else // no -> terminate
		{
			break;
		}	
	} 
	return deleted; // tell the caller the deleted element
}

void print_array(Heap h)
{
	cout << "Number of elements in heap: " << h.size << endl;
	cout << "Array:" << endl;
	if (h.size > 0)
	{
		int counter = 1;
		do
		{
			cout << "[" << h.element[counter] << "]";
			counter++;
			if (counter <= h.size)
			{
				cout << "-";
			}
		} while (counter <= h.size);
		cout << endl;
	} else
	{
		cout << "[]" << endl;
	}
}


//screen control functions

void cls()
{
	printf("%c[2J",27);
}
void setCursor(int y, int x)
{
	printf("%c[%d;%dH",27,y,x);
}
void printhline(int y, int x, int l, char n)
{
	setCursor(y,x);
	for (int i = 0; i < l/2; i++)
	{
		cout <<n;
	}
	cout<<'+';
	for (int i = 0; i < l/2; i++)
	{
		cout <<n;
	}
}
void print(Heap &heap)
{
	int max = heap.size;
	int l = 128;
	
	int line = 1;
	int step = 0;
	int dis  = 0;
	
	int n = 1;
	while (n <= max)
	{
		step = int(pow(2.0,line));
		dis  = 2*l/ int(pow(2.0,line));
		for (int i=1; i <= (step-1);  i=i+2)
		{
			int x = i*l/step; 
			int y = line*3;
			
			if(line > 1 && line < 6)		//not root
			{	
				if (i%4==1)
				{
					printhline(y-2,x+2,dis-3,'-');	
				}
			}
			if(line > 1&&i%4==1)
			{
				setCursor(y-1,x+1);
				cout<<'/';
			}
			
			if (i%4==3)
			{
				setCursor(y-1,x-1);
				cout<<'\\';	
			}
			setCursor(y,x);
			
			cout<<heap.element[n];
			n++;
			if (n>max)
			{
				break;
			}
		}
		line++;
	}
	cout <<endl;
	//cout <<"Nummer: "<<n <<endl;
	cout <<endl;
}

